import { DataSource } from '@angular/cdk/collections';
import { merge, Observable } from 'rxjs';
import { map, tap } from 'rxjs/operators';

import { MenuCategory } from '../../core/menu-category';
import { Product } from '../../core/product';

export class ProductListDataSource extends DataSource<Product> {
  public data: Product[];
  public filteredData: Product[];
  public filterValue: string;

  constructor(
    private readonly filter: Observable<string>,
    private readonly dataObs: Observable<Product[]>,
  ) {
    super();
    this.data = [];
    this.filteredData = [];
    this.filterValue = '';
  }

  connect(): Observable<Product[]> {
    const dataMutations = [
      this.dataObs.pipe(
        tap((x) => {
          this.data = x;
        }),
      ),
      this.filter.pipe(
        tap((x) => {
          this.filterValue = x;
        }),
      ),
    ];
    return merge(...dataMutations).pipe(
      map(() => this.getFilteredData(this.data, this.filterValue)),
      tap((x) => {
        this.filteredData = x;
      }),
    );
  }

  disconnect() {}

  // eslint-disable-next-line class-methods-use-this
  private getFilteredData(data: Product[], filter: string): Product[] {
    if (filter === null || filter === undefined || filter === '') {
      return data;
    }
    return data.filter((x) => (x.menuCategory as MenuCategory).id === filter);
  }
}
